/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.core.dom;

import java.util.LinkedList;
import java.util.List;

import org.eclipse.wst.xml.core.internal.provisional.document.IDOMDocument;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * XLIFF の trans-unit 要素です。
 */
@SuppressWarnings("restriction")
public class TransUnitDelegate extends TransBaseDelegate {

	private final IDOMDocument domDocument;

	public TransUnitDelegate(IDOMDocument domDocument, Element element) {
		super(element);
		this.domDocument = domDocument;
	}

	public String getId() {
		return getValue("@id");
	}

	public void setContextGroupTmOmit(boolean omit) {
		final NodeList contextGroups = element.getElementsByTagName("context-group");
		if (contextGroups.getLength() > 0) {
			final Node contextGroup = contextGroups.item(0);
			element.removeChild(contextGroup.getNextSibling());
			element.removeChild(contextGroup);
		}
		if (omit) {
			final Element contextGroup = domDocument.createElement("context-group");
			contextGroup.setAttribute("purpose", "x-tm");
			final Element context = domDocument.createElement("context");
			context.setAttribute("context-type", "x-omit");

			contextGroup.appendChild(domDocument.createTextNode("\n          "));
			contextGroup.appendChild(context);
			contextGroup.appendChild(domDocument.createTextNode("\n        "));

			insertAfter(contextGroup, getChildElement("target"), getChildElement("source"));
		}
	}

	public boolean isContextGourpTmOmit() {
		return "x-omit".equals(getValue("context-group/context/@context-type"));
	}

	public boolean setTranslate(boolean translate) {
		final String newTranslate = translate ? "yes" : "no";
		final String oldTranslate = getValue("@translate", "yes");
		if (!oldTranslate.equals(newTranslate)) {
			return setValue("@translate", newTranslate);
		}
		return false;
	}

	public boolean isTranslate() {
		return "yes".equals(getValue("@translate", "yes"));
	}

	public boolean setTarget(String target) {
		final String old = getValue("target", null);
		if (target.equals("") && old == null) {
			return false;
		}
		if (!target.equals(old)) {
			createEmptyTarget();
			return setValue("target", target);
		}
		return false;
	}

	public boolean setTargetState(String state) {
		final String oldState = getTargetState();
		if (!state.equals(oldState)) {
			createEmptyTarget();
			return setValue("target/@state", state);
		}
		return false;
	}

	public String getTargetState() {
		return getValue("target/@state");
	}

	public Node appendNote(String from, String text) {
		final Element noteElement = domDocument.createElement("note");
		final org.w3c.dom.Text textNode = domDocument.createTextNode(text);
		noteElement.appendChild(textNode);
		noteElement.setAttribute("from", from);
		final NodeList noteList = element.getElementsByTagName("note");

		Node beforeNode = null;
		if (noteList.getLength() > 0) {
			beforeNode = noteList.item(noteList.getLength() - 1);
		} else {
			beforeNode = getTargetNode();
			if (beforeNode == null) {
				beforeNode = getSourceNode();
			}
		}
		insertAfter(noteElement, beforeNode);
		return noteElement;
	}

	public void removeNote(ElementDelegate note) {
		element.removeChild(note.getElement().getNextSibling());
		element.removeChild(note.getElement());
	}

	public List<NoteDelegate> getNoteList() {
		return getList(NoteDelegate.class, "note");
	}

	public List<AltTransDelegate> getAltTransList() {
		return getList(AltTransDelegate.class, "alt-trans");
	}

	public void joinNextTransUnit() {
		final Node nextText = element.getNextSibling();
		final Element nextElement = (Element) nextText.getNextSibling();
		final TransUnitDelegate next = new TransUnitDelegate(domDocument, nextElement);

		setId(next.getId());
		setSource(joinSentence(getSource(), next.getSource()));
		createEmptyTarget();
		setTarget(joinSentence(getTarget(), next.getTarget()));

		Node beforeNode = getTargetNode();
		for (final AltTransDelegate altTrans : next.getAltTransList()) {
			final Node cloneNode = altTrans.getElement().cloneNode(true);
			insertAfter(cloneNode, beforeNode);
			beforeNode = cloneNode;
		}
		for (final NoteDelegate note : next.getNoteList()) {
			final Node cloneNode = note.getElement().cloneNode(true);
			insertAfter(cloneNode, beforeNode);
			beforeNode = cloneNode;
		}

		element.getParentNode().removeChild(nextText);
		element.getParentNode().removeChild(nextElement);
	}

	public static List<TransUnitDelegate> listOf(IDOMDocument domDocument) {
		final NodeList nodeList = domDocument.getElementsByTagName("trans-unit");
		final List<TransUnitDelegate> list = new LinkedList<TransUnitDelegate>();
		final int nodeLength = nodeList.getLength();
		for (int i = 0; i < nodeLength; i++) {
			final TransUnitDelegate ele = new TransUnitDelegate(domDocument, (Element) nodeList.item(i));
			list.add(ele);
		}
		return list;
	}

	//-------------------------------------------------------------------------
	// protected

	protected void createEmptyTarget() {
		if (getChildElement("target") == null) {
			final Element targetElement = domDocument.createElement("target");
			final org.w3c.dom.Text textNode = domDocument.createTextNode("");
			targetElement.appendChild(textNode);
			insertAfter(targetElement, getChildElement("source"));
		}
	}

	protected void insertAfter(Node insertNode, Node... beforeNodes) {
		for (final Node beforeNode : beforeNodes) {
			if (beforeNode != null) {
				final Node afterNode = beforeNode.getNextSibling().getNextSibling();

				// 次のノードが </trans-unit> の場合
				if (afterNode == null) {
					element.insertBefore(domDocument.createTextNode("  "), afterNode);
					element.insertBefore(insertNode, afterNode);
					element.insertBefore(domDocument.createTextNode("\n      "), afterNode);
				} else {
					element.insertBefore(insertNode, afterNode);
					element.insertBefore(domDocument.createTextNode("\n        "), afterNode);
				}
				return;
			}
		}
		throw new IllegalArgumentException("beforeNodes is null.");
	}

	protected boolean setSource(String source) {
		return setValue("source", source);
	}

	protected boolean setId(String id) {
		return setValue("@id", id);
	}

	protected String joinSentence(String s1, String s2) {
		if (s1.equals("") || s2.equals("")) {
			return s1 + s2;
		}
		if (s1.matches(".*\\s") || s2.matches("\\s.*")) {
			return s1 + s2;
		}
		return s1 + " " + s2;
	}
}
